@@ -27,8 +27,6 @@
 #include <sys/select.h>
 #endif
 
-#define ENABLE_CURLX_PRINTF
-/* use our own printf() functions */
 #include "curlx.h"
 
 #include "tool_cfgable.h"
@@ -57,8 +55,8 @@ size_t tool_read_cb(char *buffer, size_t sz, size_t nmemb, void *userdata)
   }
 
   if(config->timeout_ms) {
-    struct timeval now = tvnow();
-    long msdelta = tvdiff(now, per->start);
+    struct curltime now = curlx_now();
+    long msdelta = (long)curlx_timediff(now, per->start);
 
     if(msdelta > config->timeout_ms)
       /* timeout */
@@ -76,7 +74,14 @@ size_t tool_read_cb(char *buffer, size_t sz, size_t nmemb, void *userdata)
       timeout.tv_usec = (int)((wait%1000)*1000);
 
       FD_ZERO(&bits);
+#if defined(__DJGPP__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Warith-conversion"
+#endif
       FD_SET(per->infd, &bits);
+#if defined(__DJGPP__)
+#pragma GCC diagnostic pop
+#endif
       if(!select(per->infd + 1, &bits, NULL, NULL, &timeout))
         return 0; /* timeout */
     }
@@ -86,11 +91,11 @@ size_t tool_read_cb(char *buffer, size_t sz, size_t nmemb, void *userdata)
   rc = read(per->infd, buffer, sz*nmemb);
   if(rc < 0) {
     if(errno == EAGAIN) {
-      errno = 0;
+      CURL_SETERRNO(0);
       config->readbusy = TRUE;
       return CURL_READFUNC_PAUSE;
     }
-    /* since size_t is unsigned we can't return negative values fine */
+    /* since size_t is unsigned we cannot return negative values fine */
     rc = 0;
   }
   if((per->uploadfilesize != -1) &&
@@ -127,17 +132,16 @@ int tool_readbusy_cb(void *clientp,
   if(config->readbusy) {
     /* lame code to keep the rate down because the input might not deliver
        anything, get paused again and come back here immediately */
-    static long rate = 500;
-    static struct timeval prev;
+    static timediff_t rate = 500;
+    static struct curltime prev;
     static curl_off_t ulprev;
 
     if(ulprev == ulnow) {
       /* it did not upload anything since last call */
-      struct timeval now = tvnow();
+      struct curltime now = curlx_now();
       if(prev.tv_sec)
         /* get a rolling average rate */
-        /* rate = rate - rate/4 + tvdiff(now, prev)/4; */
-        rate -= rate/4 - tvdiff(now, prev)/4;
+        rate -= rate/4 - curlx_timediff(now, prev)/4;
       prev = now;
     }
     else {
@@ -154,5 +158,5 @@ int tool_readbusy_cb(void *clientp,
       tool_go_sleep(25);
   }
 
-  return per->noprogress? 0 : CURL_PROGRESSFUNC_CONTINUE;
+  return per->noprogress ? 0 : CURL_PROGRESSFUNC_CONTINUE;
 }
